==> distfun-0.3-1-src.zip: distfun/src/cdflib/bup.c <==
/* bup.f -- translated by f2c (version 20100827).
You must link the resulting object file with libf2c:
on Microsoft Windows system, link with libf2c.lib;
on Linux or Unix systems, link with .../path/to/libf2c.a -lm
or, if you install libf2c.a in a standard place, with -lf2c -lm
-- in that order, at the end of the command line, as in
cc *.o -lf2c -lm
Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,

http://www.netlib.org/f2c/libf2c.zip
*/

#include "cdflib.h"

/* Table of constant values */

static int c__1 = 1;
static int c__0 = 0;

double bup_(double *a, double *b, double *x, double *y, 
	int *n, double *eps)
{
	/* System generated locals */
	int i__1;
	double ret_val, d__1;

	/* Builtin functions */
	double exp(double);

	/* Local variables */
	static double d__;
	static int i__, k;
	static double l, r__, t, w;
	static int mu;
	static double ap1;
	static int kp1, nm1;
	static double apb;

	/* ----------------------------------------------------------------------- */
	/*     EVALUATION OF IX(A,B) - IX(A+N,B) WHERE N IS A POSITIVE INTEGER. */
	/*     EPS IS THE TOLERANCE USED. */
	/* ----------------------------------------------------------------------- */
	/*     .. Scalar Arguments .. */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */

	/*     .. */
	/*     .. Intrinsic Functions .. */
	/*     .. */
	/*     .. Executable Statements .. */

	/*          OBTAIN THE SCALING FACTOR EXP(-MU) AND */
	/*             EXP(MU)*(X**A*Y**B/BETA(A,B))/A */

	apb = *a + *b;
	ap1 = *a + 1.;
	mu = 0;
	d__ = 1.;
	if (*n == 1 || *a < 1.) {
		goto L10;
	}
	if (apb < ap1 * 1.1) {
		goto L10;
	}
	mu = (d__1 = exparg_(&c__1), (int) fabs(d__1));
	k = (int) exparg_(&c__0);
	if (k < mu) {
		mu = k;
	}
	t = (double) mu;
	d__ = exp(-t);

L10:
	ret_val = brcmp1_(&mu, a, b, x, y) / *a;
	if (*n == 1 || ret_val == 0.) {
		return ret_val;
	}
	nm1 = *n - 1;
	w = d__;

	/*          LET K BE THE INDEX OF THE MAXIMUM TERM */

	k = 0;
	if (*b <= 1.) {
		goto L50;
	}
	if (*y > 1e-4) {
		goto L20;
	}
	k = nm1;
	goto L30;
L20:
	r__ = (*b - 1.) * *x / *y - *a;
	if (r__ < 1.) {
		goto L50;
	}
	k = nm1;
	t = (double) nm1;
	if (r__ < t) {
		k = (int) r__;
	}

	/*          ADD THE INCREASING TERMS OF THE SERIES */

L30:
	i__1 = k;
	for (i__ = 1; i__ <= i__1; ++i__) {
		l = (double) (i__ - 1);
		d__ = (apb + l) / (ap1 + l) * *x * d__;
		w += d__;
		/* L40: */
	}
	if (k == nm1) {
		goto L70;
	}

	/*          ADD THE REMAINING TERMS OF THE SERIES */

L50:
	kp1 = k + 1;
	i__1 = nm1;
	for (i__ = kp1; i__ <= i__1; ++i__) {
		l = (double) (i__ - 1);
		d__ = (apb + l) / (ap1 + l) * *x * d__;
		w += d__;
		if (d__ <= *eps * w) {
			goto L70;
		}
		/* L60: */
	}

	/*               TERMINATE THE PROCEDURE */

L70:
	ret_val *= w;
	return ret_val;
} /* bup_ */

==> distfun-0.5-1-src.zip: distfun/src/cdflib/bup.c <==
/* bup.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

double cdflib_bup(double a, double b, double x, double y, 
	int n, double eps)
{
	/* System generated locals */
	int i__1;
	double ret_val, d__1;

	/* Local variables */
	static double d__;
	static int i__, k;
	static double l, r__, t, w;
	static int mu;
	static double ap1;
	static int kp1, nm1;
	static double apb;

	/* ----------------------------------------------------------------------- */
	/*     EVALUATION OF IX(A,B) - IX(A+N,B) WHERE N IS A POSITIVE INTEGER. */
	/*     EPS IS THE TOLERANCE USED. */
	/* ----------------------------------------------------------------------- */

	/*          OBTAIN THE SCALING FACTOR EXP(-MU) AND */
	/*             EXP(MU)*(X**A*Y**B/BETA(A,B))/A */

	apb = a + b;
	ap1 = a + 1.;
	mu = 0;
	d__ = 1.;
	if (n == 1 || a < 1.) {
		goto L10;
	}
	if (apb < ap1 * 1.1) {
		goto L10;
	}
	mu = (d__1 = cdflib_exparg(1), (int) fabs(d__1));
	k = (int) cdflib_exparg(0);
	if (k < mu) {
		mu = k;
	}
	t = (double) mu;
	d__ = exp(-t);

L10:
	ret_val = cdflib_brcmp1(mu, a, b, x, y) / a;
	if (n == 1 || ret_val == 0.) {
		return ret_val;
	}
	nm1 = n - 1;
	w = d__;

	/*          LET K BE THE INDEX OF THE MAXIMUM TERM */

	k = 0;
	if (b <= 1.) {
		goto L50;
	}
	if (y > 1e-4) {
		goto L20;
	}
	k = nm1;
	goto L30;
L20:
	r__ = (b - 1.) * x / y - a;
	if (r__ < 1.) {
		goto L50;
	}
	k = nm1;
	t = (double) nm1;
	if (r__ < t) {
		k = (int) r__;
	}

	/*          ADD THE INCREASING TERMS OF THE SERIES */

L30:
	i__1 = k;
	for (i__ = 1; i__ <= i__1; ++i__) {
		l = (double) (i__ - 1);
		d__ = (apb + l) / (ap1 + l) * x * d__;
		w += d__;
		/* L40: */
	}
	if (k == nm1) {
		goto L70;
	}

	/*          ADD THE REMAINING TERMS OF THE SERIES */

L50:
	kp1 = k + 1;
	i__1 = nm1;
	for (i__ = kp1; i__ <= i__1; ++i__) {
		l = (double) (i__ - 1);
		d__ = (apb + l) / (ap1 + l) * x * d__;
		w += d__;
		if (d__ <= eps * w) {
			goto L70;
		}
		/* L60: */
	}

	/*               TERMINATE THE PROCEDURE */

L70:
	ret_val *= w;
	return ret_val;
}
